#!/usr/bin/python
# coding: utf-8
#
# Copyright (c) 2009, Norfello Oy
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Norfello Oy nor the
#      names of its contributors may be used to endorse or promote products
#      derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY NORFELLO OY ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL NORFELLO OY BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


from lxml import etree
from lxml import objectify
from pkg_resources import resource_stream

from finvoicelib.error import LxmlErrorWrapper
from finvoicelib.reader.message import Message
from finvoicelib.reader.xml_reader import FinvoiceXmlReader


class Reader(object):
    """

    Finvoice file may contain more than one message.

    Message consists of:
     * SOAP-Envelop (ebxml) (optional)
     * Finvoice-xml

    """

    errors = None
    messages = None
    dtd_resource = '../resources/Finvoice.dtd'

    def __init__(self, xml):
        if isinstance(xml, basestring):
            f = open(xml)
        else:
            f = xml

        self.errors = []
        self.messages = []

        r = FinvoiceXmlReader(f)

        for block in r:
            # SOAP-ENVELOPE
            if block.soap:
                env = etree.XML(block.soap)
            else:
                env = None
            # Payload
            xml_tree = objectify.XML(block.payload)
            msg = Message(xml_tree, env)
            if not self.validate(xml_tree):
                print 'Invalid XML!'
                msg.errors += self.errors
            self.messages.append(msg)

    def validate(self, xml_tree):
        f = resource_stream(__name__, self.dtd_resource)
        dtd = etree.DTD(f)
        if dtd.validate(xml_tree):
            return True
        for error in dtd.error_log.filter_from_errors():
            self.errors.append(LxmlErrorWrapper(error))
        return False



#!/usr/bin/python
# coding: utf-8
#
# Copyright (c) 2009, Norfello Oy
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Norfello Oy nor the
#      names of its contributors may be used to endorse or promote products
#      derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY NORFELLO OY ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL NORFELLO OY BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


from finvoicelib.elements import AccountElement
from finvoicelib.elements import Element
from finvoicelib.elements import ReferenceNumberElement


class EpiAccountID(AccountElement):
    """
    EpiAccountID
    """
    tag = 'EpiAccountID'


class EpiBei(Element):
    """
    EpiBei
    """
    tag = 'EpiBei'


class EpiBfiIdentifier(Element):
    """
    EpiBfiIdentifier
    """
    tag = 'EpiBfiIdentifier'


class EpiBfiPartyDetails(Element):
    """
    EpiBfiPartyDetails
    """
    tag = 'EpiBfiPartyDetails'
    aggregate = [EpiBfiIdentifier]


class EpiCharge(Element):
    """
    EpiCharge
    """
    tag = 'EpiCharge'


class EpiDate(Element):
    """
    EpiDate
    """
    tag = 'EpiDate'


class EpiDateOptionDate(Element):
    """
    EpiDateOptionDate
    """
    tag = 'EpiDateOptionDate'


class EpiInstructedAmount(Element):
    """
    EpiInstructedAmount
    """
    tag = 'EpiInstructedAmount'


class EpiReference(Element):
    """
    EpiReference
    """
    tag = 'EpiReference'


class EpiRemittanceInfoIdentifier(ReferenceNumberElement):
    """
    EpiRemittanceInfoIdentifier
    """
    tag = 'EpiRemittanceInfoIdentifier'
    required = True


class EpiNameAddressDetails(Element):
    """
    EpiNameAddressDetails
    """
    tag = 'EpiNameAddressDetails'


class EpiBeneficiaryPartyDetails(Element):
    """
    EpiBeneficiaryPartyDetails
    """
    tag = 'EpiBeneficiaryPartyDetails'
    aggregate = [EpiNameAddressDetails, EpiBei, EpiAccountID]


class EpiIdentificationDetails(Element):
    """
    EpiIdentificationDetails
    """
    tag = 'EpiIdentificationDetails'
    aggregate = [EpiDate, EpiReference]


class EpiPartyDetails(Element):
    """
    EpiPartyDetails
    """
    tag = 'EpiPartyDetails'
    aggregate = [EpiBfiPartyDetails,
        EpiBeneficiaryPartyDetails]


class EpiPaymentInstructionDetails(Element):
    """
    EpiPaymentInstructionDetails
    """
    tag = 'EpiPaymentInstructionDetails'
    aggregate = [
        EpiRemittanceInfoIdentifier,
        EpiInstructedAmount,
        EpiCharge,
        EpiDateOptionDate, ]


class EpiDetails(Element):
    """
    EpiDetails
    """
    tag = 'EpiDetails'
    aggregate = [
        EpiIdentificationDetails,
        EpiPartyDetails,
        EpiPaymentInstructionDetails, ]



#!/usr/bin/python
# coding: utf-8
#
# Copyright (c) 2009, Norfello Oy
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Norfello Oy nor the
#      names of its contributors may be used to endorse or promote products
#      derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY NORFELLO OY ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL NORFELLO OY BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


from finvoicelib.elements import CurrencyElement
from finvoicelib.elements import DateElement
from finvoicelib.elements import Element
from finvoicelib.elements.seller import SellerReferenceIdentifier


class InvoiceDate(DateElement):
    """
    InvoiceDate
    """
    tag = 'InvoiceDate'


class InvoiceTypeCode(Element):
    """
    InvoiceTypeCode
    """
    tag = 'InvoiceTypeCode'


class InvoiceTypeText(Element):
    """
    InvoiceTypeText
    """
    tag = 'InvoiceTypeText'


class OriginCode(Element):
    """
    OriginCode
    """
    tag = 'OriginCode'


class InvoiceDueDate(DateElement):
    """
    InvoiceDueDate
    """
    tag = 'InvoiceDueDate'


class InvoiceNumber(Element):
    """
    InvoiceNumber
    """
    tag = 'InvoiceNumber'


class InvoiceSenderOrganisationName(Element):
    """
    InvoiceSenderOrganisationName
    """
    tag = 'InvoiceSenderOrganisationName'


class InvoiceSenderPartyIdentifier(Element):
    """
    InvoiceSenderPartyIdentifier
    """
    tag = 'InvoiceSenderPartyIdentifier'


class InvoiceTotalVatAmount(CurrencyElement):
    """
    InvoiceTotalVatAmount
    """
    tag = 'InvoiceTotalVatAmount'


class InvoiceTotalVatExcludedAmount(CurrencyElement):
    """
    InvoiceTotalVatExcludedAmount
    """
    tag = 'InvoiceTotalVatExcludedAmount'


class InvoiceTotalVatIncludedAmount(CurrencyElement):
    """
    InvoiceTotalVatIncludedAmount
    """
    tag = 'InvoiceTotalVatIncludedAmount'


class InvoiceTypeCode(Element):
    """
    InvoiceTypeCode
    """
    tag = 'InvoiceTypeCode'


class InvoiceTypeText(Element):
    """
    InvoiceTypeText
    """
    tag = 'InvoiceTypeText'


class InvoiceUrlNameText(Element):
    """
    InvoiceUrlNameText
    """
    tag = 'InvoiceUrlNameText'


class InvoiceUrlText(Element):
    """
    InvoiceUrlText
    """
    tag = 'InvoiceUrlText'


class VatBaseAmount(Element):
    """
    VatBaseAmount
    """
    tag = 'VatBaseAmount'


class VatRateAmount(Element):
    """
    VatRateAmount
    """
    tag = 'VatRateAmount'


class VatRatePercent(Element):
    """
    VatRatePercent
    """
    tag = 'VatRatePercent'


class VatSpecificationDetails(Element):
    """
    VatSpecificationDetails
    """
    tag = 'VatSpecificationDetails'
    aggregate = [VatBaseAmount, VatRatePercent, VatRateAmount]


class OrderIdentifier(Element):
    """
    OrderIdentifier
    """
    tag = 'OrderIdentifier'


class InvoiceSenderPartyDetails(Element):
    """
    InvoiceSenderPartyDetails
    """
    tag = 'InvoiceSenderPartyDetails'
    aggregate = [InvoiceSenderPartyIdentifier,
        InvoiceSenderOrganisationName,
        InvoiceSenderOrganisationName]


class CashDiscountAmount(Element):
    """
    CashDiscountAmount
    """
    tag = 'CashDiscountAmount'


class CashDiscountBaseAmount(Element):
    """
    CashDiscountBaseAmount
    """
    tag = 'CashDiscountBaseAmount'


class CashDiscountDate(DateElement):
    """
    CashDiscountDate
    """
    tag = 'CashDiscountDate'


class CashDiscountPercent(Element):
    """
    CashDiscountPercent
    """
    tag = 'CashDiscountPercent'


class PaymentTermsFreeText(Element):
    """
    PaymentTermsFreeText
    """
    tag = 'PaymentTermsFreeText'


class PaymentOverDueFineFreeText(Element):
    """
    PaymentOverDueFineFreeText
    """
    tag = 'PaymentOverDueFineFreeText'


class PaymentOverDueFinePercent(Element):
    """
    PaymentOverDueFinePercent
    """
    tag = 'PaymentOverDueFinePercent'


class PaymentStatusCode(Element):
    """
    PaymentStatusCode
    """
    tag = 'PaymentStatusCode'


class PaymentStatusDetails(Element):
    """
    PaymentStatusDetails
    """
    tag = 'PaymentStatusDetails'
    aggregate = [PaymentStatusCode]


class PaymentOverDueFineDetails(Element):
    """
    PaymentOverDueFineDetails
    """
    tag = 'PaymentOverDueFineDetails'
    aggregate = [PaymentOverDueFineFreeText, PaymentOverDueFinePercent]


class InvoiceFreeText(Element):
    """
    InvoiceFreeText
    """
    tag = 'InvoiceFreeText'


class PaymentTermsDetails(Element):
    """
    PaymentTermsDetails
    """
    tag = 'PaymentTermsDetails'
    aggregate = [
        PaymentTermsFreeText,
        InvoiceDueDate,
        CashDiscountDate,
        CashDiscountBaseAmount,
        CashDiscountPercent,
        CashDiscountAmount,
        PaymentOverDueFineDetails]


class InvoiceDetails(Element):
    """
    InvoiceDetails
    """
    tag = 'InvoiceDetails'
    aggregate = [
        InvoiceTypeCode,
        InvoiceTypeText,
        OriginCode,
        InvoiceNumber,
        InvoiceDate,
        InvoiceFreeText,
        SellerReferenceIdentifier,
        OrderIdentifier,
        InvoiceTotalVatExcludedAmount,
        InvoiceTotalVatAmount,
        InvoiceTotalVatIncludedAmount,
        VatSpecificationDetails,
        PaymentTermsDetails]


class VirtualBankBarcode(Element):
    """
    VirtualBankBarcode
    """
    tag = 'VirtualBankBarcode'



#!/usr/bin/python
# coding: utf-8
#
# Copyright (c) 2009, Norfello Oy
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Norfello Oy nor the
#      names of its contributors may be used to endorse or promote products
#      derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY NORFELLO OY ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL NORFELLO OY BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


from finvoicelib.elements import CurrencyElement
from finvoicelib.elements import DateElement
from finvoicelib.elements import DecimalElement
from finvoicelib.elements import Element


class ArticleIdentifier(Element):
    """
    ArticleIdentifier
    """
    tag = 'ArticleIdentifier'


class ArticleInfoUrlText(Element):
    """
    ArticleInfoUrlText
    """
    tag = 'ArticleInfoUrlText'


class ArticleName(Element):
    """
    ArticleName
    """
    tag = 'ArticleName'


class DeliveredQuantity(Element):
    """
    DeliveredQuantity
    """
    tag = 'DeliveredQuantity'

    def set_attributes(self):
        self.attributes = {'quantityunitcode': ''}


class OrderedQuantity(Element):
    """
    OrderedQuantity
    """
    tag = 'OrderedQuantity'

    def set_attributes(self):
        self.attributes = {'quantityunitcode': ''}


class ConfirmedQuantity(Element):
    """
    OrderedQuantity
    """
    tag = 'ConfirmedQuantity'

    def set_attributes(self):
        self.attributes = {'quantityunitcode': ''}


class UnitPriceAmount(CurrencyElement):
    """
    UnitPriceAmount
    """
    tag = 'UnitPriceAmount'


class RowAccountDimensionText(Element):
    """
    RowAccountDimensionText
    """
    tag = 'RowAccountDimensionText'


class RowAgreementIdentifier(Element):
    """
    RowAgreementIdentifier
    """
    tag = 'RowAgreementIdentifier'


class RowAmount(CurrencyElement):
    """
    RowAmount
    """
    tag = 'RowAmount'


class RowDelivererCountryCode(Element):
    """
    RowDelivererCountryCode
    """
    tag = 'RowDelivererCountryCode'


class RowDelivererCountryName(Element):
    """
    RowDelivererCountryName
    """
    tag = 'RowDelivererCountryName'


class RowDelivererIdentifier(Element):
    """
    RowDelivererIdentifier
    """
    tag = 'RowDelivererIdentifier'


class RowDelivererName(Element):
    """
    RowDelivererName
    """
    tag = 'RowDelivererName'


class RowDeliveryDate(DateElement):
    """
    RowDeliveryDate
    """
    tag = 'RowDeliveryDate'


class RowDiscountPercent(Element):
    """
    RowDiscountPercent
    """
    tag = 'RowDiscountPercent'


class RowFreeText(Element):
    """
    RowFreeText
    """
    tag = 'RowFreeText'


class RowIdentifier(Element):
    """
    RowIdentifier
    """
    tag = 'RowIdentifier'


class RowIdentifierDate(DateElement):
    """
    RowIdentifierDate
    """
    tag = 'RowIdentifierDate'


class RowManufacturerCountryCode(Element):
    """
    RowManufacturerCountryCode
    """
    tag = 'RowManufacturerCountryCode'


class RowManufacturerCountryName(Element):
    """
    RowManufacturerCountryName
    """
    tag = 'RowManufacturerCountryName'


class RowManufacturerIdentifier(Element):
    """
    RowManufacturerIdentifier
    """
    tag = 'RowManufacturerIdentifier'


class RowManufacturerName(Element):
    """
    RowManufacturerName
    """
    tag = 'RowManufacturerName'


class RowNormalProposedAccountIdentifier(Element):
    """
    RowNormalProposedAccountIdentifier
    """
    tag = 'RowNormalProposedAccountIdentifier'


class RowPriceListIdentifier(Element):
    """
    RowPriceListIdentifier
    """
    tag = 'RowPriceListIdentifier'


class RowRequestOfQuotationIdentifier(Element):
    """
    RowRequestOfQuotationIdentifier
    """
    tag = 'RowRequestOfQuotationIdentifier'


class RowShortProposedAccountIdentifier(Element):
    """
    RowShortProposedAccountIdentifier
    """
    tag = 'RowShortProposedAccountIdentifier'


class RowSubIdentifier(Element):
    """
    RowSubIdentifier
    """
    tag = 'RowSubIdentifier'


class RowVatAmount(CurrencyElement):
    """
    RowVatAmount
    """
    tag = 'RowVatAmount'


class RowVatExcludedAmount(CurrencyElement):
    """
    RowVatExcludedAmount
    """
    tag = 'RowVatExcludedAmount'


class RowVatRatePercent(DecimalElement):
    """
    RowVatRatePercent
    """
    tag = 'RowVatRatePercent'


class RowWaybillIdentifier(Element):
    """
    RowWaybillIdentifier
    """
    tag = 'RowWaybillIdentifier'


class RowDeliveryDetails(Element):
    """
    RowDeliveryDetails
    """
    tag = 'RowDeliveryDetails'
    aggregate = [
        RowWaybillIdentifier,
        RowDelivererIdentifier,
        RowDelivererName,
        RowDelivererName,
        RowDelivererCountryCode,
        RowDelivererCountryName,
        RowManufacturerIdentifier,
        RowManufacturerName,
        RowManufacturerCountryCode,
        RowManufacturerCountryName]


class InvoiceRow(Element):
    """
    InvoiceRow
    """
    tag = 'InvoiceRow'
    aggregate = [
        RowSubIdentifier,
        ArticleIdentifier,
        ArticleName,
        DeliveredQuantity,
        OrderedQuantity,
        ConfirmedQuantity,
        UnitPriceAmount,
        RowAccountDimensionText,
        RowIdentifier,
        RowIdentifierDate,
        RowDeliveryDate,
        RowAgreementIdentifier,
        RowRequestOfQuotationIdentifier,
        RowPriceListIdentifier,
        RowDeliveryDetails,
        RowShortProposedAccountIdentifier,
        RowNormalProposedAccountIdentifier,
        RowFreeText,
        RowVatRatePercent,
        RowVatAmount,
        RowVatExcludedAmount,
        RowAmount]



#!/usr/bin/python
# coding: utf-8
#
# Copyright (c) 2009, Norfello Oy
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Norfello Oy nor the
#      names of its contributors may be used to endorse or promote products
#      derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY NORFELLO OY ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL NORFELLO OY BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


from finvoicelib.elements import DateElement
from finvoicelib.elements import Element
from finvoicelib.elements.country import CountryName


class ManufacturerCountryCode(Element):
    """
    ManufacturerCountryCode
    """
    tag = 'ManufacturerCountryCode'


class ManufacturerCountryName(Element):
    """
    ManufacturerCountryName
    """
    tag = 'ManufacturerCountryName'


class ManufacturerIdentifier(Element):
    """
    ManufacturerIdentifier
    """
    tag = 'ManufacturerIdentifier'


class ManufacturerName(Element):
    """
    ManufacturerName
    """
    tag = 'ManufacturerName'


class DelivererCountryCode(Element):
    """
    DelivererCountryCode
    """
    tag = 'DelivererCountryCode'


class DelivererCountryName(Element):
    """
    DelivererCountryName
    """
    tag = 'DelivererCountryName'


class DelivererIdentifier(Element):
    """
    DelivererIdentifier
    """
    tag = 'DelivererIdentifier'


class DelivererName(Element):
    """
    DelivererName
    """
    tag = 'DelivererName'


class DeliveryDate(DateElement):
    """
    DeliveryDate
    """
    tag = 'DeliveryDate'


class DeliveryMethodText(Element):
    """
    DeliveryMethodText
    """
    tag = 'DeliveryMethodText'


class DeliveryOrganisationName(Element):
    """
    DeliveryOrganisationName
    """
    tag = 'DeliveryOrganisationName'


class DeliveryPartyIdentifier(Element):
    """
    DeliveryPartyIdentifier
    """
    tag = 'DeliveryPartyIdentifier'


class DeliveryPostCodeIdentifier(Element):
    """
    DeliveryPostCodeIdentifier
    """
    tag = 'DeliveryPostCodeIdentifier'


class DeliveryPostofficeBoxIdentifier(Element):
    """
    DeliveryPostofficeBoxIdentifier
    """
    tag = 'DeliveryPostofficeBoxIdentifier'


class DeliveryStreetName(Element):
    """
    DeliveryStreetName
    """
    tag = 'DeliveryStreetName'


class DeliveryTermsText(Element):
    """
    DeliveryTermsText
    """
    tag = 'DeliveryTermsText'


class TerminalAddressText(Element):
    """
    TerminalAddressText
    """
    tag = 'TerminalAddressText'


class WaybillIdentifier(Element):
    """
    WaybillIdentifier
    """
    tag = 'WaybillIdentifier'


class WaybillTypeCode(Element):
    """
    WaybillTypeCode
    """
    tag = 'WaybillTypeCode'


class DeliveryTownName(Element):
    """
    DeliveryTownName
    """
    tag = 'DeliveryTownName'


class DeliveryEmailaddressIdentifier(Element):
    """
    DeliveryEmailaddressIdentifier
    """
    tag = 'DeliveryEmailaddressIdentifier'


class DeliveryPhoneNumberIdentifier(Element):
    """
    DeliveryPhoneNumberIdentifier
    """
    tag = 'DeliveryPhoneNumberIdentifier'


class DeliveryCommunicationDetails(Element):
    """
    DeliveryCommunicationDetails
    """
    tag = 'DeliveryCommunicationDetails'
    aggregate = [DeliveryEmailaddressIdentifier,
        DeliveryPhoneNumberIdentifier]


class DeliveryContactPersonName(Element):
    """
    DeliveryContactPersonName
    """
    tag = 'DeliveryContactPersonName'


class DeliveryContactPersonEmail(Element):
    """
    DeliveryContactPersonName
    """
    tag = 'DeliveryContactPersonEmail'


class DeliveryPostalAddressDetails(Element):
    """
    DeliveryPostalAddressDetails
    """
    tag = 'DeliveryPostalAddressDetails'
    aggregate = [DeliveryStreetName,
        DeliveryTownName,
        DeliveryPostCodeIdentifier,
        CountryName,
        DeliveryPostofficeBoxIdentifier]


class DeliveryPartyDetails(Element):
    """
    DeliveryPartyDetails
    """
    tag = 'DeliveryPartyDetails'
    aggregate = [DeliveryPartyIdentifier,
        DeliveryOrganisationName,
        DeliveryPostalAddressDetails]


class DeliveryDetails(Element):
    """
    DeliveryDetails
    """
    tag = 'DeliveryDetails'
    aggregate = [DeliveryDate,
        DeliveryMethodText,
        DeliveryTermsText,
        TerminalAddressText,
        WaybillIdentifier,
        WaybillTypeCode,
        DelivererIdentifier,
        DelivererName,
        DelivererName,
        DelivererCountryCode,
        DelivererCountryName,
        ManufacturerIdentifier,
        ManufacturerName,
        ManufacturerCountryCode,
        ManufacturerCountryName]



#!/usr/bin/python
# coding: utf-8
#
# Copyright (c) 2009, Norfello Oy
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Norfello Oy nor the
#      names of its contributors may be used to endorse or promote products
#      derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY NORFELLO OY ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL NORFELLO OY BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


from finvoice.elements import Element


class SubOriginalInvoiceNumber(Element):
    """
    SubOriginalInvoiceNumber
    """
    tag = 'SubOriginalInvoiceNumber'


class SubRowAccountDimensionText(Element):
    """
    SubRowAccountDimensionText
    """
    tag = 'SubRowAccountDimensionText'


class SubRowAgreementIdentifier(Element):
    """
    SubRowAgreementIdentifier
    """
    tag = 'SubRowAgreementIdentifier'


class SubRowAmount(Element):
    """
    SubRowAmount
    """
    tag = 'SubRowAmount'


class SubRowDelivererCountryCode(Element):
    """
    SubRowDelivererCountryCode
    """
    tag = 'SubRowDelivererCountryCode'


class SubRowDelivererCountryName(Element):
    """
    SubRowDelivererCountryName
    """
    tag = 'SubRowDelivererCountryName'


class SubRowDelivererIdentifier(Element):
    """
    SubRowDelivererIdentifier
    """
    tag = 'SubRowDelivererIdentifier'


class SubRowDelivererName(Element):
    """
    SubRowDelivererName
    """
    tag = 'SubRowDelivererName'


class SubRowDeliveryDate(Element):
    """
    SubRowDeliveryDate
    """
    tag = 'SubRowDeliveryDate'


class SubRowDeliveryIdentifier(Element):
    """
    SubRowDeliveryIdentifier
    """
    tag = 'SubRowDeliveryIdentifier'


class SubRowIdentifier(Element):
    """
    SubRowIdentifier
    """
    tag = 'SubRowIdentifier'


class SubRowIdentifierDate(Element):
    """
    SubRowIdentifierDate
    """
    tag = 'SubRowIdentifierDate'


class SubRowManufacturerCountryCode(Element):
    """
    SubRowManufacturerCountryCode
    """
    tag = 'SubRowManufacturerCountryCode'


class SubRowManufacturerCountryName(Element):
    """
    SubRowManufacturerCountryName
    """
    tag = 'SubRowManufacturerCountryName'


class SubRowManufacturerIdentifier(Element):
    """
    SubRowManufacturerIdentifier
    """
    tag = 'SubRowManufacturerIdentifier'


class SubRowManufacturerName(Element):
    """
    SubRowManufacturerName
    """
    tag = 'SubRowManufacturerName'


class SubRowNormalProposedAccountIdentifier(Element):
    """
    SubRowNormalProposedAccountIdentifier
    """
    tag = 'SubRowNormalProposedAccountIdentifier'


class SubRowPriceListIdentifier(Element):
    """
    SubRowPriceListIdentifier
    """
    tag = 'SubRowPriceListIdentifier'


class SubRowRequestOfQuotationIdentifier(Element):
    """
    SubRowRequestOfQuotationIdentifier
    """
    tag = 'SubRowRequestOfQuotationIdentifier'


class SubRowShortProposedAccountIdentifier(Element):
    """
    SubRowShortProposedAccountIdentifier
    """
    tag = 'SubRowShortProposedAccountIdentifier'


class SubRowVatAmount(Element):
    """
    SubRowVatAmount
    """
    tag = 'SubRowVatAmount'


class SubRowVatExcludedAmount(Element):
    """
    SubRowVatExcludedAmount
    """
    tag = 'SubRowVatExcludedAmount'


class SubRowVatRatePercent(Element):
    """
    SubRowVatRatePercent
    """
    tag = 'SubRowVatRatePercent'


class SubRowWaybillIdentifier(Element):
    """
    SubRowWaybillIdentifier
    """
    tag = 'SubRowWaybillIdentifier'


class SubArticleIdentifier(Element):
    """
    SubArticleIdentifier
    """
    tag = 'SubArticleIdentifier'


class SubArticleName(Element):
    """
    SubArticleName
    """
    tag = 'SubArticleName'


class SubRowDeliveryDetails(Element):
    """
    SubRowDeliveryDetails
    """
    tag = 'SubRowDeliveryDetails'
    aggregate = [
        SubRowWaybillIdentifier,
        SubRowDelivererIdentifier,
        SubRowDelivererName,
        SubRowDelivererName,
        SubRowDelivererCountryCode,
        SubRowDelivererCountryName,
        SubRowManufacturerIdentifier,
        SubRowManufacturerName,
        SubRowManufacturerCountryCode,
        SubRowManufacturerCountryName, ]


class SubInvoiceRow(Element):
    """
    SubInvoiceRow
    """
    tag = 'SubInvoiceRow'
    aggregate = [
        SubArticleIdentifier,
        SubArticleName,
        SubRowIdentifier,
        SubRowIdentifierDate,
        SubOriginalInvoiceNumber,
        SubRowDeliveryIdentifier,
        SubRowDeliveryDate,
        SubRowAgreementIdentifier,
        SubRowRequestOfQuotationIdentifier,
        SubRowPriceListIdentifier,
        SubRowDeliveryDetails,
        SubRowShortProposedAccountIdentifier,
        SubRowNormalProposedAccountIdentifier,
        SubRowAccountDimensionText,
        SubRowVatRatePercent,
        SubRowVatAmount,
        SubRowVatExcludedAmount,
        SubRowAmount]



#!/usr/bin/python
# coding: utf-8
#
# Copyright (c) 2009, Norfello Oy
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Norfello Oy nor the
#      names of its contributors may be used to endorse or promote products
#      derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY NORFELLO OY ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL NORFELLO OY BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


from lxml import etree


class SoapEnvelope(object):

    seller_id = None
    seller_intermediator_id = None
    buyer_id = None
    buyer_intermediator_id = None

    def __init__(self, tree):

        ns = tree.nsmap.get('eb')

        def build_xpath(elem, role):
            return ('//{%(N)s}%(elem)s/{%(N)s}'
                    'PartyId[../{%(N)s}Role'
                    ' = "%(role)s"]/text()'
                    % {'N': ns, 'elem': elem, 'role': role})

        # Functions for finding net service ids
        get_seller_id = etree.ETXPath(build_xpath('From', 'Sender'))
        get_seller_intermed_id = \
            etree.ETXPath(build_xpath('From', 'Intermediator'))

        get_buyer_id = etree.ETXPath(build_xpath('To', 'Receiver'))
        get_buyer_intermed_id = \
            etree.ETXPath(build_xpath('To', 'Intermediator'))

        seller_id = get_seller_id(tree)
        if len(seller_id)>0:
            self.seller_id = str(seller_id[0])

        seller_intermediator_id = get_seller_intermed_id(tree)
        if len(seller_intermediator_id)>0:
            self.seller_intermediator_id = seller_intermediator_id[0]

        buyer_id = get_buyer_id(tree)
        if len(buyer_id)>0:
            self.buyer_id = str(buyer_id[0])

        buyer_intermediator_id = get_buyer_intermed_id(tree)
        if len(buyer_intermediator_id)>0:
            self.buyer_intermediator_id = buyer_intermediator_id[0]



#!/usr/bin/python
# coding: utf-8
#
# Copyright (c) 2009, Norfello Oy
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Norfello Oy nor the
#      names of its contributors may be used to endorse or promote products
#      derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY NORFELLO OY ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL NORFELLO OY BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import datetime
from decimal import Decimal

from finvoicelib.reader import Reader
from finvoicelib.tests import FinvoiceTestCase
from finvoicelib.wrapper import FinvoiceWrapper


class TestFinvoiceWrapperWithPulliFinvoice(FinvoiceTestCase):

    """
    TestFinvoiceWrapper(WithPulliFinvoice)

    Contains the test suite for the module finvoice.wrapper and acts as a
    documentation for the FinvoiceWrapper.

    """

    def setUp(self):
        """
        Set up the FinvoiceWrapper

        FinvoiceWrapper essentially wraps the most used elements of the
        finvoice payload to a convinient class.
        """

        # Reader only accepts file-like objects
        reader = Reader(self.get_pulli_finvoice())

        # Get first message from the file.
        msg = reader.messages[0]

        self.wrapper = FinvoiceWrapper(msg)

    def test_payee_name(self):
        """
        test_payee_name

        payee.name carries the value from the first <SellerOrganisationName>
        element.

        """
        payee = self.wrapper.payee
        self.assertEqual(payee.name, 'Pullin Musiikki Oy')

    def test_payee_business_id(self):
        """
        test_payee_business_id

        payee.business_id carries the business id of the seller party. This
        field can be empty if the business_id wasn't declared or it had
        a invalid value.

        """
        payee = self.wrapper.payee
        self.assertEqual(str(payee.business_id), '0199920-7')

    def test_payee_bank_account_number(self):
        """
        test_payee_bank_account_number
        """
        payee = self.wrapper.payee
        self.assertEqual(str(payee.bank_account_number), '500015-20000081')

    def test_payee_address(self):
        """
        test_payee_address

        payee.postal_address contain the collected values from the
        SellerPostalAddressDetails aggregate.

        """
        address = self.wrapper.payee.postal_address
        self.assertEqual(address.street_name, 'Puukatu 2 F')
        self.assertEqual(address.postal_code, '00112')
        self.assertEqual(address.city, 'HELSINKI')
        self.assertEqual(address.country, '')

    def test_payee_fax(self):
        """
        test_payee_bank_account_number
        """
        payee = self.wrapper.payee
        self.assertEqual(payee.fax, '(09) 1232500')

    def test_payee_bank_swift_code(self):
        """
        test_payee_bank_swift_code

        """
        payee = self.wrapper.payee
        self.assertEqual(str(payee.bank_swift_code), 'OKOYFIHH')

    def test_payee_contact_email(self):
        """
        test_payee_contact_email
        """
        payee = self.wrapper.payee
        self.assertEqual(payee.contact_email,
            'hanna.paananen@pullinmusiikki.fi')

    def test_payee_contact_name(self):
        """
        test_payee_contact_name
        """
        payee = self.wrapper.payee
        self.assertEqual(payee.contact_name, 'Hanna Paananen')

    def test_payee_contact_phone(self):
        """
        test_payee_contact_phone
        """
        payee = self.wrapper.payee
        self.assertEqual(payee.contact_phone, '09-3213212')

    def test_payee_iban_account_number(self):
        """
        test_payee_iban_account_number

        """
        # TODO: unimplemented
        payee = self.wrapper.payee
        self.assertEqual(payee.iban_account_number, '')

    def test_payee_website(self):
        """
        test_payee_contact_phone
        """
        payee = self.wrapper.payee
        self.assertEqual(payee.website, 'www.pullinmusiikki.fi')

    def test_payee_invoice_recipient_einvoice_address(self):
        """
        test_payee_invoice_recipient_einvoice_address
        """
        invoice_recipient = self.wrapper.invoice_recipient
        self.assertEqual(invoice_recipient.einvoice_address, \
                         'FI2757800750155448')

    def test_payee_invoice_recipient_intermediator_address(self):
        """
        test_payee_invoice_recipient_intermediator_address
        """
        invoice_recipient = self.wrapper.invoice_recipient
        self.assertEqual(invoice_recipient.intermediator_address, \
                         'OKOYFIHH')

    def test_payee_organisation_unit_number(self):
        """
        test_payee_contact_phone
        """
        payee = self.wrapper.payee
        self.assertEqual(payee.organisation_unit_number, '003701999207')

    def test_payee_organisation_taxcode(self):
        """
        test_payee_contact_phone
        """
        payee = self.wrapper.payee
        self.assertEqual(payee.organisation_taxcode, '0199920-7')

    def test_payee_bank_accounts(self):
        """
        test_payee_contact_phone
        """
        payee = self.wrapper.payee
        self.assertEqual(len(payee.bank_accounts), 3)

        self.assertEqual(payee.bank_accounts[0].account_number,
                         'FI2757800750155448')
        self.assertEqual(payee.bank_accounts[0].swift_code, 'OKOYFIHH')

        self.assertEqual(payee.bank_accounts[1].account_number,
                         'FI2721221222212227')
        self.assertEqual(payee.bank_accounts[1].swift_code, 'NDEAFIHH')

        self.assertEqual(payee.bank_accounts[2].account_number,
                         'FI2781232323312334')
        self.assertEqual(payee.bank_accounts[2].swift_code, 'PSPBFIHH')

    def test_payer_name(self):
        """
        test_payer_name

        payer.name carries the value from the first <BuyerOrganisationName>
        element.

        """
        payer = self.wrapper.payer
        self.assertEqual(payer.name, 'Sensorit Oy')

    def test_payer_net_service_id(self):
        """
        test_payer_net_service_id

        payer.net_service_id contains the buyer's party id from the
        SOAP-ENVELOPE.

        """
        payer = self.wrapper.payer
        self.assertEqual(payer.net_service_id, '')
        self.assertEqual(payer.intermediator, None)

    def test_payer_business_id(self):
        """
        test_payer_business_id

        payer.business_id carries the business id of the buyer party. This
        field can be empty if the business id wasn't declared or it had
        a invalid value.

        """
        payer = self.wrapper.payer
        self.assertEqual(str(payer.business_id), '1234567-1')

    def test_payer_address(self):
        """
        test_payer_address

        payer.postal_address contain the collected values from the
        BuyerPostalAddressDetails aggregate.
        """
        address = self.wrapper.payer.postal_address
        self.assertEqual(address.street_name, 'Sempalokatu 2')
        self.assertEqual(address.postal_code, '00122')
        self.assertEqual(address.city, 'HELSINKI')
        self.assertEqual(address.country, '')

    def test_payer_contact_name(self):
        """
        test_payer_contact_name
        """
        payer = self.wrapper.payer
        self.assertEqual(payer.contact_name, 'Hannes Puumalainen')

    def test_payer_contact_email(self):
        """
        test_payer_contact_email

        """
        payer = self.wrapper.payer
        self.assertEqual(str(payer.contact_email),
            'hannes.puumalainen@sensorit.fi')

    def test_payer_customer_number(self):
        """
        test_payer_customer_number
        """
        payer = self.wrapper.payer
        self.assertEqual(payer.customer_number, '1122')

    def test_payer_contact_phone(self):
        """
        test_payer_contact_phone

        """
        payer = self.wrapper.payer
        self.assertEqual(str(payer.contact_phone), '050-543 2658')

    def test_payer_organisation_unit_number(self):
        """
        test_payer_contact_phone

        """
        payer = self.wrapper.payer
        self.assertEqual(str(payer.organisation_unit_number), '00371234567')

    def test_payer_organisation_taxcode(self):
        """
        test_payer_contact_phone

        """
        payer = self.wrapper.payer
        self.assertEqual(str(payer.organisation_taxcode), 'FI12345671')

    def test_invoice_total_taxed(self):
        """
        test_invoice_total_taxed

        invoice.total_taxed carries the value from the element
        <InvoiceTotalVatIncludedAmount>.

        invoice.total_taxed_currency has the currency code declared in the
        element by the attribute AmountCurrencyIdentifier.

        """
        invoice = self.wrapper.invoice
        self.assertEqual(invoice.total_taxed, Decimal('3352.98'))
        self.assertEqual(invoice.total_taxed_currency, 'EUR')

    def test_invoice_total_untaxed(self):
        """
        test_invoice_total_untaxed

        invoice.total_untaxed carries the value from the element
        <InvoiceTotalVatExcludedAmount>.

        invoice.total_untaxed_currency has the currency code declared in the
        element by the attribute AmountCurrencyIdentifier.

        """
        invoice = self.wrapper.invoice
        self.assertEqual(invoice.total_untaxed, Decimal('2830.30'))
        self.assertEqual(invoice.total_untaxed_currency, 'EUR')

    def test_invoice_vat_amount(self):
        """
        test_invoice_vat_amount

        invoice.vat_amount carries the value from the element
        <InvoiceTotalVatAmount>.

        invoice.vat_amount_currency has the currency code declared in the
        element by the attribute AmountCurrencyIdentifier.

        """
        invoice = self.wrapper.invoice
        self.assertEqual(invoice.vat_amount, Decimal('622.68'))
        self.assertEqual(invoice.vat_amount_currency, 'EUR')

    def test_invoice_number(self):
        """
        test_invoice_number
        """
        invoice = self.wrapper.invoice
        self.assertEqual(invoice.number, '159')

    def test_invoice_free_text(self):
        """
        test_invoice_free_text

        invoice.free_text has the combined values from all the
        <InvoiceFreeText> elements.

        """
        invoice = self.wrapper.invoice
        self.assertEqual(invoice.free_text,
                         u'Vapaata tekstiä enintään 512 merkkiä')

    def test_invoice_overdue_percent(self):
        """
        test_invoice_overdue_percent
        """
        invoice = self.wrapper.invoice
        self.assertEqual(invoice.payment_overdue_percent, '16')

    def test_invoice_overdue_freetext(self):
        """
        test_invoice_overdue_freetext

        Element: <PaymentOverDueFineFreeText>
        """
        invoice = self.wrapper.invoice
        self.assertEqual(invoice.payment_overdue_text, u'Viivästyskorko 16%')

    def test_invoice_terms_of_payment(self):
        """
        test_invoice_terms_of_payment

        """
        invoice = self.wrapper.invoice
        self.assertEqual(invoice.terms_of_payment, (u'5 pävää ./.2%,'
                                                    u' 14 päivää netto'))

    def test_invoice_typecode(self):
        """
        test_invoice_typecode

        """
        invoice = self.wrapper.invoice
        self.assertEqual(invoice.typecode, u'INV01')

    def test_invoice_type_text(self):
        """
        test_invoice_type_text

        """
        invoice = self.wrapper.invoice
        self.assertEqual(invoice.type_text, u'LASKU')

    def test_rows_len(self):
        """
        test_rows_len

        All the <InvoiceRow> -elements are collected to the list: rows.

        """
        rows = self.wrapper.rows
        self.assertEqual(len(rows), 8)

    def test_rows_0(self):
        """
        test_rows_0

        Demonstrates the available attibutes of a row.
        """
        row = self.wrapper.rows[0]
        self.assertEqual(row.name, 'TUURA')
        self.assertEqual(row.article_identifier, '123123')
        self.assertEqual(row.quantity, Decimal('10'))
        self.assertEqual(row.unit_price, Decimal('28.69'))
        self.assertEqual(row.total_taxed, Decimal("350.02"))
        self.assertEqual(row.total_taxed_currency, 'EUR')
        self.assertEqual(row.total_untaxed, Decimal('286.90'))
        self.assertEqual(row.total_untaxed_currency, 'EUR')
        self.assertEqual(row.vat_amount, Decimal("63.12"))
        self.assertEqual(row.vat_amount_currency, 'EUR')
        self.assertEqual(row.vat_percent, Decimal('22'))
        self.assertEqual(row.unit_code, 'kpl')
        self.assertEqual(row.freetext,
            u'Tuote myydään varsien ja ohjeen kanssa.\n')
        self.assertEqual(row.date, datetime.date(2004, 12, 13))
        self.assertEqual(row.account_dimension_text, None)

    def test_rows_1(self):
        """
        test_rows_1

        Demonstrates the available attibutes of a row.
        """
        row = self.wrapper.rows[1]
        self.assertEqual(row.name, 'Tussi')
        self.assertEqual(row.article_identifier, '8768217637')
        self.assertEqual(row.quantity, Decimal('120'))
        self.assertEqual(row.unit_price, Decimal('1.64'))
        self.assertEqual(row.total_taxed, Decimal("240.10"))
        self.assertEqual(row.total_taxed_currency, 'EUR')
        self.assertEqual(row.total_untaxed, Decimal('196.80'))
        self.assertEqual(row.total_untaxed_currency, 'EUR')
        self.assertEqual(row.vat_amount, Decimal("43.30"))
        self.assertEqual(row.vat_amount_currency, 'EUR')
        self.assertEqual(row.vat_percent, Decimal('22'))
        self.assertEqual(row.unit_code, 'kpl')
        self.assertEqual(row.account_dimension_text, None)


    def test_rows_2(self):
        """
        test_rows_1

        Demonstrates the available attibutes of a row.
        """
        row = self.wrapper.rows[2]
        self.assertEqual(row.name, '')
        self.assertEqual(row.article_identifier, '')
        self.assertEqual(row.quantity, None)
        self.assertEqual(row.unit_price, None)
        self.assertEqual(row.total_taxed, None)
        self.assertEqual(row.total_taxed_currency, None)
        self.assertEqual(row.total_untaxed, None)
        self.assertEqual(row.total_untaxed_currency, None)
        self.assertEqual(row.vat_amount, None)
        self.assertEqual(row.vat_amount_currency, None)
        self.assertEqual(row.vat_percent, None)
        self.assertEqual(row.unit_code, None)
        self.assertEqual(row.account_dimension_text, None)

    def test_rows_5(self):
        """
        test_rows_1

        Demonstrates the available attibutes of a row.
        """
        row = self.wrapper.rows[5]
        self.assertEqual(row.name, 'Mikseri')
        self.assertEqual(row.article_identifier, '44213213')
        self.assertEqual(row.quantity, 12)
        self.assertEqual(row.unit_price, Decimal('192.62'))
        self.assertEqual(row.total_untaxed, Decimal('2265.21'))
        self.assertEqual(row.total_untaxed_currency, 'EUR')
        self.assertEqual(row.vat_amount, Decimal('498.35'))
        self.assertEqual(row.vat_amount_currency, 'EUR')
        self.assertEqual(row.vat_percent, Decimal('22'))
        self.assertEqual(row.unit_code, 'kpl')
        self.assertEqual(row.account_dimension_text, '4500')


    def test_delivery_party_contact_email(self):
        """
        test_invoice_delivery_contact_email

        invoice.delivery_contact_email contains the value from the element
        <DeliveryEmailaddressIdentifier>.
        """
        delivery_party = self.wrapper.delivery_party
        self.assertEqual(delivery_party.contact_email,
            'maija.vilkkunen@kolumbus.fi')

    def test_delivery_party_contact_name(self):
        """
        test_invoice_delivery_contact_email

        invoice.delivery_contact_name contains the value from the element
        <DeliveryContactName>.
        """
        delivery_party = self.wrapper.delivery_party
        self.assertEqual(delivery_party.contact_name, 'Maija Vilkkunen')

    def test_delivery_party_contact_phone(self):
        """
        test_invoice_delivery_contact_phone

        invoice.delivery_contact_phone contains the value from the element
        <DeliveryPhoneNumberIdentifier>.

        """
        delivery_party = self.wrapper.delivery_party
        self.assertEqual(delivery_party.contact_phone, '09-123 12345')

    def test_delivery_party_delivery_date(self):
        """
        test_invoice_delivery_date

        invoice.delivery_contact_phone contains the value from the element
        <DeliveryDate>.

        """
        delivery_party = self.wrapper.delivery_party
        self.assertEqual(delivery_party.delivery_date,
            datetime.date(2004, 12, 5))

    def test_delivery_party_delivery_method(self):
        """
        test_invoice_delivery_date

        """
        delivery_party = self.wrapper.delivery_party
        self.assertEqual(delivery_party.delivery_method, 'Noudetaan')

    def test_delivery_party_name(self):
        """
        test_invoice_delivery_organisation_name

        invoice.delivery_contact_phone contains the value from the
        <DeliveryOrganisationName> element.
        """
        delivery_party = self.wrapper.delivery_party
        self.assertEqual(delivery_party.name, u'Helsingin Tanssihalli')

    def test_delivery_party_address(self):
        """
        test_delivery_party_address

        delivery_party.address contain the collected values from the
        DeliveryPostalAddressDetails aggregate.
        """
        delivery_address = self.wrapper.delivery_party.address
        self.assertEqual(delivery_address.street_name, 'Satamakatu 2')
        self.assertEqual(delivery_address.postal_code, '00100')
        self.assertEqual(delivery_address.city, 'Helsinki')
        self.assertEqual(delivery_address.country, '')
        self.assertEqual(delivery_address.post_office_box, '')

    def test_delivery_party_name(self):
        """
        test_invoice_delivery_organisation_name

        invoice.delivery_contact_phone contains the value from the
        <DeliveryOrganisationName> element.
        """
        invoice_recipient = self.wrapper.invoice_recipient
        self.assertEqual(invoice_recipient.organisation_name, u'Tilitoimisto Ryynänen')

    def test_invoice_recipient_party_address(self):
        """
        test_delivery_party_address

        delivery_party.address contain the collected values from the
        DeliveryPostalAddressDetails aggregate.
        """
        invoice_recipient = self.wrapper.invoice_recipient.address
        self.assertEqual(invoice_recipient.street_name, u'Mäkelänkatu 2')
        self.assertEqual(invoice_recipient.postal_code, '00102')
        self.assertEqual(invoice_recipient.city, 'Helsinki')
        self.assertEqual(invoice_recipient.country, 'FINLAND')
        self.assertEqual(invoice_recipient.post_office_box, 'PL 22')

class TestFinvoiceWrapperWithPankkiyhdistysFinvoice(FinvoiceTestCase):
    """
    TestFinvoiceWrapper(WithPankkiyhdistysFinvoice)

    Contains the test suite for the module finvoice.wrapper and acts as a
    documentation for the FinvoiceWrapper.

    """

    def setUp(self):
        """
        Set up the FinvoiceWrapper

        FinvoiceWrapper essentially wraps the most used elements of the
        finvoice payload to a convinient class.
        """

        # Reader only accepts file-like objects
        reader = Reader(self.get_pankkiyhdistys_finvoice())

        # Get first message from the file.
        msg = reader.messages[0]

        self.wrapper = FinvoiceWrapper(msg)

    def test_payee_name(self):
        """
        test_payee_name

        payee.name carries the value from the first <SellerOrganisationName>
        element.

        """
        payee = self.wrapper.payee
        self.assertEqual(payee.name, 'Pullin Kala')

    def test_payee_business_id(self):
        """
        test_payee_business_id

        payee.business_id carries the business id of the seller party. This
        field can be empty if the business_id wasn't declared or it had
        a invalid value.

        """
        payee = self.wrapper.payee
        self.assertEqual(payee.business_id, '')

    def test_payer_name(self):
        """
        test_payer_name

        payer.name carries the value from the first <BuyerOrganisationName>
        element.

        """
        payer = self.wrapper.payer
        self.assertEqual(payer.name, 'Kalakauppa Vilkkunen')

    def test_payer_net_service_id(self):
        """
        test_payer_net_service_id
        """
        payer = self.wrapper.payer
        self.assertEqual(payer.net_service_id, 'FI3329501800008512')
        self.assertEqual(payer.intermediator, 'NDEAFIHH')

    def test_payee_net_service_id(self):
        """
        test_payee_net_service_id
        """
        payee = self.wrapper.payee
        self.assertEqual(payee.net_service_id, 'FI9859292720000267')
        self.assertEqual(payee.intermediator, 'OKOYFIHH')



#!/usr/bin/python
# coding: utf-8
#
# Copyright (c) 2009, Norfello Oy
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Norfello Oy nor the
#      names of its contributors may be used to endorse or promote products
#      derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY NORFELLO OY ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL NORFELLO OY BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


from finvoicelib.elements import BusinessIdElement
from finvoicelib.elements import Element
from finvoicelib.elements.country import CountryCode
from finvoicelib.elements.country import CountryName


class SellerOrganisationUnitNumber(Element):
    """
    SellerOrganisationUnitNumber
    """
    tag = 'SellerOrganisationUnitNumber'


class SellerPhoneNumberIdentifier(Element):
    """
    SellerPhoneNumberIdentifier
    """
    tag = 'SellerPhoneNumberIdentifier'


class SellerVatRegistrationDate(Element):
    """
    SellerVatRegistrationDate
    """
    tag = 'SellerVatRegistrationDate'


class SellerVatRegistrationText(Element):
    """
    SellerVatRegistrationText
    """
    tag = 'SellerVatRegistrationText'


class SellerAccountID(Element):
    """
    SellerAccountID
    """
    tag = 'SellerAccountID'


class SellerBic(Element):
    """
    SellerBic
    """
    tag = 'SellerBic'


class SellerAccountDetails(Element):
    """
    SellerAccountDetails
    """
    tag = 'SellerAccountDetails'
    aggregate = [SellerAccountID, SellerBic]


class SellerCommonEmailaddressIdentifier(Element):
    """
    SellerCommonEmailaddressIdentifier
    """
    tag = 'SellerCommonEmailaddressIdentifier'


class SellerContactPersonName(Element):
    """
    SellerContactPersonName
    """
    tag = 'SellerContactPersonName'


class SellerEmailaddressIdentifier(Element):
    """
    SellerEmailaddressIdentifier
    """
    tag = 'SellerEmailaddressIdentifier'


class SellerCommunicationDetails(Element):
    """
    SellerCommunicationDetails
    """
    tag = 'SellerCommunicationDetails'
    aggregate = [SellerEmailaddressIdentifier,
        SellerPhoneNumberIdentifier]


class SellerFaxNumber(Element):
    """
    SellerFaxNumber
    """
    tag = 'SellerFaxNumber'


class SellerFreeText(Element):
    """
    SellerFreeText
    """
    tag = 'SellerFreeText'


class SellerHomeTownName(Element):
    """
    SellerHomeTownName
    """
    tag = 'SellerHomeTownName'


class SellerOrganisationName(Element):
    """
    SellerOrganisationName
    """
    tag = 'SellerOrganisationName'


class SellerOrganisationTaxCode(Element):
    """
    SellerOrganisationTaxCode
    """
    tag = 'SellerOrganisationTaxCode'

class SellerOrganisationTaxCodeUrlText(Element):
    """
    SellerOrganisationTaxCodeUrlText
    """
    tag = 'SellerOrganisationTaxCodeUrlText'


class SellerPartyIdentifier(BusinessIdElement):
    """
    SellerPartyIdentifier
    """
    tag = 'SellerPartyIdentifier'


class SellerPhoneNumber(Element):
    """
    SellerPhoneNumber
    """
    tag = 'SellerPhoneNumber'


class SellerPostCodeIdentifier(Element):
    """
    SellerPostCodeIdentifier
    """
    tag = 'SellerPostCodeIdentifier'


class SellerReferenceIdentifier(Element):
    """
    SellerReferenceIdentifier
    """
    tag = 'SellerReferenceIdentifier'


class SellerStreetName(Element):
    """
    SellerStreetName
    """
    tag = 'SellerStreetName'


class SellerTownName(Element):
    """
    SellerTownName
    """
    tag = 'SellerTownName'


class SellerWebaddressIdentifier(Element):
    """
    SellerWebaddressIdentifier
    """
    tag = 'SellerWebaddressIdentifier'


class SellerPostalAddressDetails(Element):
    """
    SellerPostalAddressDetails
    """
    tag = 'SellerPostalAddressDetails'
    aggregate = [SellerStreetName,
        SellerTownName,
        SellerPostCodeIdentifier,
        CountryCode,
        CountryName]


class SellerPartyDetails(Element):
    """
    SellerPartyDetails
    """
    tag = 'SellerPartyDetails'
    aggregate = [SellerPartyIdentifier,
        SellerOrganisationName,
        SellerOrganisationTaxCode,
        SellerOrganisationTaxCodeUrlText,
        SellerPostalAddressDetails]


class InvoiceRecipientOrganisationName(Element):
    """
    InvoiceRecipientOrganisationName
    """
    tag = 'InvoiceRecipientOrganisationName'


class InvoiceRecipientPartyIdentifier(Element):
    """
    InvoiceRecipientPartyIdentifier
    """
    tag = 'InvoiceRecipientPartyIdentifier'


class InvoiceRecipientOrganisationTaxCode(Element):
    """
    InvoiceRecipientOrganisationName
    """
    tag = 'InvoiceRecipientOrganisationTaxCode'


class InvoiceRecipientPostCodeIdentifier(Element):
    """
    InvoiceRecipientPostCodeIdentifier
    """
    tag = 'InvoiceRecipientPostCodeIdentifier'


class InvoiceRecipientPostOfficeBoxIdentifier(Element):
    """
    InvoiceRecipientPostOfficeBoxIdentifier
    """
    tag = 'InvoiceRecipientPostOfficeBoxIdentifier'


class InvoiceRecipientStreetName(Element):
    """
    InvoiceRecipientStreetName
    """
    tag = 'InvoiceRecipientStreetName'


class InvoiceRecipientTownName(Element):
    """
    InvoiceRecipientTownName
    """
    tag = 'InvoiceRecipientTownName'


class InvoiceRecipientAddress(Element):
    """
    InvoiceRecipientAddress
    """
    tag = 'InvoiceRecipientAddress'


class InvoiceRecipientIntermediatorAddress(Element):
    """
    InvoiceRecipientIntermediatorAddress
    """
    tag = 'InvoiceRecipientIntermediatorAddress'


class InvoiceRecipientDetails(Element):
    """
    InvoiceRecipientDetails
    """
    tag = 'InvoiceRecipientDetails'
    aggregate = [InvoiceRecipientAddress,
        InvoiceRecipientIntermediatorAddress]


class SellerInformationDetails(Element):
    """
    SellerInformationDetails
    """
    tag = 'SellerInformationDetails'
    aggregate = [SellerHomeTownName,
        SellerPhoneNumber,
        SellerFaxNumber,
        SellerCommonEmailaddressIdentifier,
        SellerWebaddressIdentifier,
        SellerFreeText,
        SellerAccountDetails,
        SellerAccountDetails,
        SellerAccountDetails,
        InvoiceRecipientDetails,
        SellerVatRegistrationText,
        SellerVatRegistrationDate]


class InvoiceRecipientPostalAddressDetails(Element):
    """
    InvoiceRecipientPostalAddressDetails
    """
    tag = 'InvoiceRecipientPostalAddressDetails'
    aggregate = [InvoiceRecipientStreetName,
        InvoiceRecipientTownName,
        InvoiceRecipientPostCodeIdentifier,
        CountryCode,
        CountryName,
        InvoiceRecipientPostOfficeBoxIdentifier]


class InvoiceRecipientPartyDetails(Element):
    """
    InvoiceRecipientPartyDetails
    """
    tag = 'InvoiceRecipientPartyDetails'
    aggregate = [InvoiceRecipientPartyIdentifier,
        InvoiceRecipientOrganisationName,
        InvoiceRecipientOrganisationTaxCode,
            InvoiceRecipientPostalAddressDetails, ]



#!/usr/bin/python
# coding: utf-8
#
# Copyright (c) 2009, Norfello Oy
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Norfello Oy nor the
#      names of its contributors may be used to endorse or promote products
#      derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY NORFELLO OY ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL NORFELLO OY BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from setuptools import setup, find_packages

version = '0.1.11'


setup(name='FinvoiceLib',
      author='Norfello Oy',
      author_email='opensource@norfello.com',
      version=version,
      description="A library for reading Finvoice XML files",
      long_description="""
Finvoice is one of most commonly used XML dialects used for electronic
invoicing in Finland.

One of the major drawbacks of this format has been the lack of commonly
available libraries for reading this format. This is exactly why Norfello
developed an open source library for reading the format. With this library you
can easily incorporate the receiving of Finvoice based messages to your
software.

FinvoiceLib consists of two main components: Reader and FinvoiceWrapper.
The Reader component is the heart of the library as it encapsulates the full
finvoice 1.2 message to a class based structure. This class structure can then
be used access every single element found in the message. The second component,
FinvoiceWrapper, wraps the most commonly used elements to a convinient helper
class, that allows to access any of available elements with only a one single
line of code!
      """,
      classifiers=['Development Status :: 3 - Alpha',
                   'Intended Audience :: Developers',
                   'Intended Audience :: Financial and Insurance Industry',
                   'License :: OSI Approved :: BSD License',
                   'Programming Language :: Python',
                   'Topic :: Office/Business', ],
      keywords='finvoice norfello postita.fi',
      url='http://en.norfello.com/',
      license='BSD',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
      'lxml',
      'business_tools>=0.2.12'
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



#!/usr/bin/python
# coding: utf-8
#
# Copyright (c) 2009, Norfello Oy
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Norfello Oy nor the
#      names of its contributors may be used to endorse or promote products
#      derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY NORFELLO OY ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL NORFELLO OY BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import time

import datetime
from business_tools import BBAN
from business_tools import IBAN
from business_tools import BusinessID
from business_tools import ReferenceNumber
from decimal import Decimal
from finvoicelib.error import ElementMissingError
from finvoicelib.error import InvalidValue
from finvoicelib.error import UnknownAttributeWarning
from finvoicelib.error import UnknownElementWarning
from finvoicelib.mapping import Map


class Element(object):

    required = False
    tag = None
    tag_ns = None
    children = None
    text = None
    node = None
    supported_elements = []
    aggregate = []
    errors = None
    attributes = None

    def __repr__(self):
        return "<Element: %s>" % self.tag

    def __init__(self, tag=None):
        if tag:
            self.tag = tag
        if not self.tag_ns:
            self.tag_ns = ''
        self.errors = []
        self.children = []
        self.text = ''
        self.node = None
        self.set_children()
        self.set_attributes()
        self._set_supported_elements()

    def set_aggregate(self):
        self.aggregate = []

    def set_children(self):
        self.children = []
        for node in self.aggregate:
            self.children.append(Map(node))

    def set_attributes(self):
        self.attributes = {}

    def _set_supported_elements(self):
        self.supported_elements = []
        for child in self.children:
            self.supported_elements.append(child.tag)

    def get_children_by_name(self, name):
        for child in self.children:
            if child.tag == name:
                return child.collection
        return None

    def has_mapping_for(self, tag):
        if self.tag_ns:
            tag = tag.replace(self.tag_ns, '')
        if tag in self.supported_elements:
            return True
        return False

    def _validate(self):
        # Implemented by subclass
        return True

    def add_node(self, node):
        tag = node.tag
        if self.tag_ns:
            tag = tag.replace(self.tag_ns, '')
        if tag not in self.supported_elements:
            self.errors.append(UnknownElementWarning(node))
            return

        t = [(i, e.tag) for i, e in enumerate(self.children) if e.tag == tag]
        if len(t) < 1:
            return

        t = t[0]
        i = t[0]
        e = self.children[i]
        obj = e.cls(node.tag)
        obj.build(node)
        self.children[i].add(obj)

    def __getattr__(self, attr):
        if attr in self.attributes:
            return self.attributes[attr]

        if not attr.endswith('Set'):
            raise AttributeError(attr)

        attr = attr[:-3]

        t = [(i, e.tag) for i, e in \
            enumerate(self.__dict__['children']) if e.tag == attr]

        if len(t) < 1:
            raise AttributeError(attr)
        t = t[0]

        return self.__dict__['children'][t[0]]

    def build_children(self, tree):
        for node in tree.iterchildren():
            tag = node.tag
            if tag == self.tag:
                continue
            if self.has_mapping_for(tag):
                self.add_node(node)
            else:
                self.errors.append(UnknownElementWarning(node))

    def build(self, tree):
        for attr in tree.attrib:
            normalized_attr = attr.lower()
            if normalized_attr in self.attributes:
                self.attributes[normalized_attr] = tree.attrib[attr]
            else:
                self.errors.append(UnknownAttributeWarning(tree, attr))

        self.set_node(tree)
        self.set_text(tree.text)
        self.build_children(tree)
        self.validate()

    def get_errors(self, error_type):
        if error_type not in ('WARNING', 'ERROR'):
            raise Exception('Unknown error type!')

        my_errors = [err for err in self.errors \
                        if err.error_type == error_type]

        my_errors += self.get_child_errors(error_type)
        return my_errors

    def get_child_errors(self, error_type):
        errors = []
        for map in self.children:
            for child in map:
                errors += child.get_errors(error_type)
        return errors

    def validate(self):
        counters = dict([(key, 0) for key in self.supported_elements])
        for mapper in self.children:
            for e in mapper.collection:
                counters[e.tag] += 1

        for element in self.aggregate:
            if element.required and counters[element.tag] < 1:
                self.errors.append(ElementMissingError(element))

    def set_text(self, value):
        self.text = value

    def set_node(self, value):
        self.node = value


class DateElement(Element):

    text = None

    def set_attributes(self):
        self.attributes = {'format': 'CCYYMMDD'}

    def set_text(self, value):
        format = self.attributes['format']
        if len(format) < 1:
            self.errors.append(InvalidValue(self.node, value))
            self.format = 'CCYYMMDD'

        format = format.replace('CCYY', r'%Y')
        format = format.replace('YYYY', r'%Y')
        format = format.replace('CCCC', r'%Y') # hack
        format = format.replace('MM', r'%m')
        format = format.replace('DD', r'%d')

        try:
            self.text = datetime.date(*time.strptime(value, format)[0:3])
        except (ValueError, TypeError), e:
            self.errors.append(InvalidValue(self.node, value))
            self.text = None


class BusinessIdElement(Element):

    def set_text(self, value):
        self.text = None
        try:
            self.text = BusinessID(value)
        except ValueError, e:
            self.errors.append(InvalidValue(self.node, value))


class ReferenceNumberElement(Element):

    def set_text(self, value):
        self.text = None
        try:
            if ReferenceNumber.is_valid(value):
                self.text = ReferenceNumber(value)
        except ValueError, e:
            self.errors.append(InvalidValue(self.node, value))


class DecimalElement(Element):

    def set_text(self, value):
        try:
            if isinstance(value, basestring):
                value = value.replace(',', '.')
            self.text = Decimal(value)
        except ValueError, e:
            self.errors.append(InvalidValue(self.node, value))


class CurrencyElement(Element):

    def set_attributes(self):
        self.attributes = {'amountcurrencyidentifier': ''}


class AccountElement(Element):

    def set_text(self, value):
        if not value:
            self.errors.append(InvalidValue(self.node, value))
            return

        if len(value)<1:
            self.errors.append(InvalidValue(self.node, value))
            return

        if ord(value[0])>60:
            try:
                self.text = IBAN(value)
            except ValueError, e:
                self.errors.append(InvalidValue(self.node, value))
        else:
            try:
                self.text = BBAN(value)
            except ValueError, e:
                self.errors.append(InvalidValue(self.node, value))
